function [kap1,kap2,psi,d] = inflation(alpha,phi,beta,sigf,sigv,sigu,theta,delta,tau,epsi,g,labor)

%OUTPUT

%this computes the reduced form inflation process for Woodford's chapter 5
%model with capital

%  pie(t) = beta*E[pie(t+1)|information at t] + d*marginal cost(t)

%kap1 and kap2 are parameters in the decision rule for the ith firm's
%capital: k_t+1(i) = kap1*k_t(i) + kap2*p_t(i),
%where k_t+1(i) is the capital stock (in deviation from the aggregate - see
%Woodford's tilde notation) chosen by the ith intermediate good firm at
%time t and p_t(i) is the ith firm's price level at time t (whether chosen
%optimally or not).

%INPUT:

%production function = k*f(h/k), k is capital and h is labor
%utility function, u(c) - int(0,1)v(h(j)), where int(0,1) indicates
%integration with respect to j, from j=0 to j=1.
% Y=[int(0,1) y(j)^((theta-1)/theta)]^(theta/(theta-1)), Y is final output and y(j) is intermediate output
% I = I(k(t+1)/k(t))k(t), where I is investment and I(.) is the cost of
% adjustment function with I(1)=delta, I'(1)=1, and I''(1)=epsi.

%alpha = Calvo probability of not changing prices
%sigf = -f''*hb/f', hb=h/k, in steady state
%sigu = -u''*c/u, in steady state
%sigv = v''h/v', in steady state
%beta = discount rate
%theta = elasticity of substitution parameter (see above)
%delta parameter of investment adjustment cost function (see above)
%epsi curvature of adjustment cost function (see above)
%phi = f/(f'*hb), in steady state

%if labor = 1, assume local labor market. Otherwise, economy-wide
if labor ~= 1
    sigv=0;
end


%%%%%%%%%%%%%%
%set up some parameters.....
omegp=sigf*phi;
omeg=sigv*phi+omegp;

rhoy=sigv*phi+omegp*phi/(phi-1);
rhok=rhoy-sigv;

rho=(1/beta)+delta-1;%steady state shadow rent on capital

k=(1+tau)/(rho*(theta/(theta-1))*(phi/(phi-1)));%steady state capital stock
c=1-delta*k-g;%ss consumption to output ratio

if c < 0
    error('fatal (inflation) negative steady state consumption to output ratio')
end

sig=c/sigu;
pphi=1+beta+(1-beta*(1-delta))*rhok/epsi;
KSI=(1-beta*(1-delta))*rhoy*theta/epsi;
%%%%%%%%%%%%%%

%first step: find a good initial guess for the solution, and also investigate
%uniqueness

kkap1=-1:.01:.99;
for ii = 1:length(kkap1)
    kap1=kkap1(ii);
    [ff,zap2,zpsi] = newsolve(kap1,KSI,alpha,beta,omeg,sigv,pphi,theta);
    fzz(ii)=ff;
end

ix=1;
ffx=fzz(1);
for ii = 2:length(kkap1)
    if ffx*fzz(ii) < 0
        II(ix)=ii;
        ix=ix+1;        
    end
    ffx=fzz(ii);
end

if ix > 2 
    ix
    kkap1(II)
    plot(kkap1,fzz,kkap1(II(1)),0,'*',kkap1(II(2)),0,'*',kkap1,zeros(size(kkap1)))
    ylabel('Criterion')
    xlabel('\kappa _{1}')
    title('Multiple Solutions')
    %print -deps c:\sw20\docs\referee\eggertsson-woodford\figurea
    error('fatal (inflation) multiple solutions for kap1')
end

if ix == 1 
    error('fatal (inflation) no solution for kap1')
end

%step 2: now refine the guess...
I1=II(1)-1;
I2=II(1);

options = optimset('LevenbergMarquardt','off','Display','off','TolFun',1e-10,'TolX',1e-10,'MaxFun',2000,'Diagnostics','off','MaxFunEvals',2000);
[kap1,fval,exitflag]=fzero(@newsolve,[kkap1(I1),kkap1(I2)],options,KSI,alpha,beta,omeg,sigv,pphi,theta);

if exitflag == 0
    disp('(inflation) maximum number of function evaluations reached')
elseif exitflag > 0
    disp('(inflation) converged to a solution')
elseif exitflag < 0
    disp('(inflation) did not converge to a solution')
end

if isnan(fval) == 1 | abs(imag(fval)) > .1e-0 
    error('fatal (inflation) optimized function value not a sensible number')
end

%now fill in the other details of the solution...
[kap2] = getkap2(KSI,alpha,kap1,beta);
[psi] = getpsi(omeg,sigv,alpha,beta,theta,kap1,kap2);
[gamma,d] = getd(alpha,beta,kap1,kap2,theta,omeg,sigv);

%verify that the equations to be solved indeed are solved...no message is
%printed if everything is ok
[err] = check(pphi,alpha,psi,beta,kap1,kap2,KSI,omeg,sigv,theta);


ip=0;
if ip == 1;%this lets you look at the criterion function for kap1, to see where the solution is and whether there is just one.
    plot(kkap1,fzz,kap1,0,'*')
    ylabel('Criterion')
    xlabel('\kappa _{1}')
    title('Investigating Uniqueness of Solution')
    %print -deps c:\sw20\docs\referee\eggertsson-woodford\figure
end
